
//Thais Moreira    Feb 2020
//Assume all files in raw folder has three channels. only analyze 3rd channel -- far-red channel 

run("Colors...", "foreground=white background=black selection=yellow");
run("Options...", "iterations=1 count=1 black");
run("Set Measurements...", "area mean integrated redirect=None decimal=2");

rawdir=getDirectory("User Choose Raw Data Folder");
resultdir=getDirectory("User Choose Result Data Folder");
list=getFileList(rawdir);

//setBatchMode(true);

var pixelscale, threshold_method, puncta_minsize, puncta_maxsize, factor;
// puncta max diameter in um unit 
var punctadiameter = 20;

parameter_input();

print("RawDataFolder:	"+rawdir);
print("puncta_MinSize(um^2)	puncta_MaxSize(um^2)	ThresholdMethod	Threshold_Factor");
print(puncta_minsize+"	"+puncta_maxsize+"	"+threshold_method+"	"+factor);
print("FileName	puncta#	TotalSize(um^2)	AverageSize(um^2)");
selectWindow("Log");
saveAs("Text", resultdir+"Summary.xls");
selectWindow("Log"); run("Close");

for(f=0;f<list.length;f++)
 {
  run("Bio-Formats Importer", "open=["+rawdir+list[f]+"] color_mode=Default rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
  splitchannel();

  process(); 
 }

function process()
 {
  run("Duplicate...", "title=copy");
 	
  //subtract bacground
  radius = punctadiameter / pixelscale;
  run("Subtract Background...", "rolling=30");
  
  run("Gaussian Blur...", "sigma=2");

  setAutoThreshold(threshold_method);
  //run("Find Maxima...", "prominence=5 above output=[Segmented Particles]");
  getThreshold(lower, upper);
  setThreshold(lower*factor, upper);
  setOption("BlackBackground", true);
  run("Convert to Mask");
  run("Fill Holes");
  run("Watershed");

  area=0; count=0;
  run("Analyze Particles...", "size="+puncta_minsize+"-"+puncta_maxsize+" circularity=0.0-1 exclude clear add");
  if( roiManager("Count") > 0 )
   {
    roiManager("Save", resultdir+list[f]+"_PunctaROI.zip");
    selectWindow("raw"); roiManager("Measure");
    selectWindow("Results");  saveAs("Text", resultdir+list[f]+"_rawresult.xls"); 

    count=roiManager("Count"); 
    for(i=0;i<count;i++)
     area = area + getResult("Area", i);
   }
  run("Clear Results"); roiManager("reset"); run("Close All");
  string=list[f]+"	"+count+"	"+area+"	"+area/count; 
  File.append(string, resultdir+"Summary.xls");
 }

function splitchannel()
 {
  getPixelSize(dummy, pixelscale, pixelscale);
  run("Split Channels");
  selectWindow("C2-"+list[f]); close();
  selectWindow("C1-"+list[f]); close();
  selectWindow("C3-"+list[f]); rename("raw");
 }


function parameter_input()
 {
  Dialog.create("Parameter_input");
  Dialog.addNumber("puncta MinSize (um^2):", 2);
  Dialog.addNumber("puncta MaxSize (um^2):", 50);
  Dialog.addChoice("Threshold Type:", newArray("Otsu dark", "Triangle dark", "Huang dark", "Default dark", "Yen dark", "Sahnbhag dark", "Li dark"));
  Dialog.addNumber("Thresholding Factor:", 1);
  
  Dialog.show();

  puncta_minsize = Dialog.getNumber();
  puncta_maxsize = Dialog.getNumber();
  threshold_method = Dialog.getChoice();
  factor = Dialog.getNumber();
 }

